@@ -8,10 +8,14 @@
  * Port to openSSL-1.1 by Marcus Meissner, 11/2017
  */
 
-#include <openssl/evp.h>
-#include <assert.h>
+#define OPENSSL_API_COMPAT 0x10100000L
 #include "aes.h"
 #include "aes_ossl.h"
+#ifdef HAVE_OPENSSL_EVP_H
+#include <openssl/evp.h>
+#endif
+#include <assert.h>
+#include "secmem.h"
 #include <string.h>
 
 #include <netinet/in.h>
@@ -26,6 +30,7 @@
  *   which is currently only done in the test_aes benchmark.
  *   So if it breaks, it won'd break regular dd_rescue usage.
  * - We have two asserts in the code to avoid silent breakage.
+ * - We fall back to calling EVP_Cipher_Init() for 3.0+ to be safe.
  */
 struct _evp_cipher_ctx_st {
     const EVP_CIPHER *cipher;
@@ -40,20 +45,44 @@ struct _evp_cipher_ctx_st {
     /* FIXME: Should this even exist? It appears unused */
     void *app_data;             /* application stuff */
     int key_len;                /* May change for variable length cipher */
+#if OPENSSL_VERSION_MAJOR >= 3 && (OPENSSL_VERSION_MINOR > 0 || OPENSSL_VERSION_PATCH >= 6)
+    int iv_len;                 /* IV length */
+#endif
     unsigned long flags;        /* Various flags */
     void *cipher_data;          /* per EVP data */
     int final_used;
     int block_mask;
     unsigned char final[EVP_MAX_BLOCK_LENGTH]; /* possible final block */
+#if OPENSSL_VERSION_MAJOR >= 3
+    /*
+     * Opaque ctx returned from a providers cipher algorithm implementation
+     * OSSL_FUNC_cipher_newctx()
+     */
+    void *algctx;
+    EVP_CIPHER *fetched_cipher;
+#endif
 } /* EVP_CIPHER_CTX */ ;
 
 
+#if OPENSSL_VERSION_MAJOR >= 3
+#define SET_IV(x, buf, sz, enc) EVP_CipherInit(x, NULL, NULL, buf, enc)
+#else
+#define SET_IV(x, buf, sz, enc) do { memcpy((void*)EVP_CIPHER_CTX_original_iv((x)), buf, sz); \
+				     memcpy(EVP_CIPHER_CTX_iv_noconst((x)), buf, sz); } while(0)
+#endif
+#if OPENSSL_VERSION_MAJOR >= 3
+#define GET_IV(x, buf, sz) EVP_CIPHER_CTX_get_updated_iv((x), buf, sz)
+#else
+#define GET_IV(x, buf, sz) memcpy(buf, EVP_CIPHER_CTX_iv((x)), sz)
+#endif
 /* An awful hack to directly access fields in EVP_CIPHER_CTX */
 void AES_OSSL_Recycle(unsigned char* ctx)
 {
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;
 	struct _evp_cipher_ctx_st *ectx = (struct _evp_cipher_ctx_st*)*evpctx;
-	assert(ectx->oiv == EVP_CIPHER_CTX_original_iv(*evpctx));
+	//fprintf(stderr, "Ectx %p, original_iv %p, offset %zx\n", evpctx, EVP_CIPHER_CTX_original_iv(*evpctx),
+	//		offsetof(struct _evp_cipher_ctx_st, oiv));
+	//assert(ectx->oiv == EVP_CIPHER_CTX_original_iv(*evpctx));
 	assert(ectx->cipher_data == EVP_CIPHER_CTX_get_cipher_data(*evpctx));
 	ectx->buf_len = 0;
 	ectx->num = 0;
@@ -99,38 +128,38 @@ int AES_OSSL_##BITCHAIN##_Encrypt(const unsigned char* ctx, unsigned int rounds,
 {								\
 	int olen, elen, ores;					\
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;	\
-	EVP_CIPHER_CTX_set_padding(evpctx[0], DOPAD? pad: 0);	\
 	if (IV) {						\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[0]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[0]), iv, 16);	\
+		SET_IV(evpctx[0], iv, 16, 1);			\
 	}							\
+	EVP_CIPHER_CTX_set_padding(evpctx[0], DOPAD? pad: 0);	\
+	if (!len && !pad) { *flen = 0; return 0; }		\
        	if (DOPAD && !pad && (len&15)) {			\
 		ores = EVP_EncryptUpdate(evpctx[0], out, &olen, in, len-(len&15));	\
 		assert(ores);					\
-		uchar ibf[16];					\
+		uchar *ibf = crypto->blkbuf2;			\
 		memcpy(ibf, in+olen, len&15);			\
 		memset(ibf+(len&15), 0, 16-(len&15));		\
 		ores = EVP_EncryptUpdate(evpctx[0], out+olen, &elen, ibf, 16);	\
 		memset(ibf, 0, len&15);				\
-		asm("":::"memory");				\
+		LFENCE;						\
 		assert(ores);					\
 	} else {								\
-		if (DOPAD && !(len%15) && pad == PAD_ASNEEDED)	\
+		if (DOPAD && !(len%16) && pad == PAD_ASNEEDED)			\
 			EVP_CIPHER_CTX_set_padding(evpctx[0], 0);		\
 		ores = EVP_EncryptUpdate(evpctx[0], out, &olen, in, len);	\
 		assert(ores);					\
-		ores = EVP_EncryptFinal(evpctx[0], out+olen, &elen);\
+		ores = EVP_EncryptFinal(evpctx[0], out+olen, &elen);		\
 		assert(ores);					\
 		if (0 && elen && (len&15)) olen -= 16;		\
 	}							\
 	*flen = olen+elen;					\
-	if (DOPAD && pad == PAD_ASNEEDED && !(len&15))		\
+	if (0 && DOPAD && pad == PAD_ASNEEDED && !(len&15))			\
 		*flen -= 16;					\
 	if (0 && olen+elen < len)				\
 		fprintf(stderr, "Encryption length mismatch %i+%i != %zi\n",	\
 			olen, elen, len);			\
 	if (IV)							\
-		memcpy(iv, EVP_CIPHER_CTX_iv(evpctx[0]), 16);			\
+		GET_IV(evpctx[0], iv, 16);			\
 	return (DOPAD && (pad == PAD_ALWAYS || (len&15)))? 16-(len&15): 0;	\
 };								\
 int AES_OSSL_##BITCHAIN##_Decrypt(const unsigned char* ctx, unsigned int rounds,\
@@ -141,14 +170,14 @@ int AES_OSSL_##BITCHAIN##_Decrypt(const unsigned char* ctx, unsigned int rounds,
 	int olen, elen = 0, ores;				\
 	int ilen = (len&15)? len+15-(len&15): len;		\
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;	\
-	EVP_CIPHER_CTX_set_padding(evpctx[0], DOPAD && pad != PAD_ASNEEDED?pad:0);	\
 	if (IV) {						\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[0]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[0]), iv, 16);	\
+		SET_IV(evpctx[0], iv, 16, 0);			\
 	}							\
+	EVP_CIPHER_CTX_set_padding(evpctx[0], DOPAD && pad != PAD_ASNEEDED?pad:0);	\
+	if (!len && pad != PAD_ALWAYS) { *flen = 0; return 0; }	\
 	if (DOPAD && pad == PAD_ASNEEDED) {			\
 		int olen1;					\
-		uchar buf[16];					\
+		uchar *buf = crypto->blkbuf3;			\
 		ores = EVP_DecryptUpdate(evpctx[0], out, &olen, in, ilen-16);	\
 		assert(ores);					\
 		EVP_CIPHER_CTX *ctx2 = EVP_CIPHER_CTX_new();	\
@@ -170,21 +199,23 @@ int AES_OSSL_##BITCHAIN##_Decrypt(const unsigned char* ctx, unsigned int rounds,
 			ores = EVP_DecryptFinal(evpctx[0], out+olen, &elen);	\
 			assert(ores);				\
 		}						\
+		/* if (elen) *(out+olen+elen) = 16-elen; */	\
 		EVP_CIPHER_CTX_free(ctx2);			\
-		asm("":::"memory");				\
+		LFENCE;						\
 	} else {						\
 		ores = EVP_DecryptUpdate(evpctx[0], out, &olen, in, ilen);	\
 		assert(ores);					\
 		ores = EVP_DecryptFinal(evpctx[0], out+olen, &elen);	\
+		/* if (elen) *(out+olen+elen) = 16-elen; */	\
 	}							\
 	if (DOPAD && pad) {					\
 		*flen = olen + elen;				\
 	} else							\
 		*flen = len;					\
 	if (IV)							\
-		memcpy(iv, EVP_CIPHER_CTX_iv(evpctx[0]), 16);	\
+		GET_IV(evpctx[0], iv, 16);			\
 	if (DOPAD && pad == PAD_ASNEEDED)			\
-		return (elen? 16-elen: 1);			\
+		return (elen? 16-elen: ILLEGAL_PADDING);	\
 	return ores - 1;					\
 }
 
@@ -249,7 +280,7 @@ void AES_OSSL_Bits_EKey_ExpandX2(const EVP_CIPHER *cipher, const unsigned char*
 	EVP_CipherInit_ex(evpctx[1], cipher, NULL, usrkey2, NULL, ENCRYPT);
 	//EVP_CIPHER_CTX_set_padding(evpctx+1, 0);
 	memset(usrkey2, 0, 32);
-	asm("":::"memory");
+	LFENCE;
 }
 void AES_OSSL_Bits_DKey_ExpandX2(const EVP_CIPHER *cipher, const unsigned char* userkey, unsigned char *ctx, unsigned int bits)
 {
@@ -271,7 +302,7 @@ void AES_OSSL_Bits_DKey_ExpandX2(const EVP_CIPHER *cipher, const unsigned char*
 	EVP_CipherInit_ex(evpctx[1], cipher, NULL, usrkey2, NULL, DECRYPT);
 	//EVP_CIPHER_CTX_set_padding(evpctx[1], 0);
 	memset(usrkey2, 0, 32);
-	asm("":::"memory");
+	LFENCE;
 }
 
 void AES_OSSL_RecycleX2(unsigned char *ctx)
@@ -307,23 +338,22 @@ int  AES_OSSL_##BITCHAIN##_EncryptX2(const unsigned char* ctx, unsigned int roun
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;	\
 	/* EVP_EncryptInit(evpctx[0], NULL, NULL, NULL);	\
 	EVP_EncryptInit(evpctx[1], NULL, NULL, NULL); */	\
+	if (IV) {						\
+		SET_IV(evpctx[0], iv, 16, 1);			\
+		SET_IV(evpctx[1], iv, 16, 1);			\
+	}							\
 	EVP_CIPHER_CTX_set_padding(evpctx[0], pad);		\
 	EVP_CIPHER_CTX_set_padding(evpctx[1], 0);		\
-	if (IV) {						\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[0]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[0]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[1]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[1]), iv, 16);	\
-	}							\
+	if (!len && !pad) { *flen = 0; return 0; }		\
        	if (!pad && (len&15)) {					\
 		ores = EVP_EncryptUpdate(evpctx[0], out, &olen, in, len-(len&15));	\
 		assert(ores);					\
-		uchar ibf[16];					\
+		uchar *ibf = crypto->blkbuf2;			\
 		memcpy(ibf, in+olen, len&15);			\
 		memset(ibf+(len&15), 0, 16-(len&15));		\
 		ores = EVP_EncryptUpdate(evpctx[0], out+olen, &elen, ibf, 16);		\
 		memset(ibf, 0, len&15);				\
-		asm("":::"memory");				\
+		LFENCE;						\
 		assert(ores);					\
 	} else {						\
 		ores = EVP_EncryptUpdate(evpctx[0], out, &olen, in, len);	\
@@ -339,7 +369,7 @@ int  AES_OSSL_##BITCHAIN##_EncryptX2(const unsigned char* ctx, unsigned int roun
 	if (pad == PAD_ASNEEDED && !(len&15))			\
 		*flen -= 16;					\
 	if (IV)							\
-		memcpy(iv, EVP_CIPHER_CTX_iv(evpctx[0]), 16);	\
+		GET_IV(evpctx[0], iv, 16);			\
 	return (pad == PAD_ALWAYS || (len&15))? 16-(len&15): 0;	\
 };								\
 int  AES_OSSL_##BITCHAIN##_DecryptX2(const unsigned char* ctx, unsigned int rounds,	\
@@ -350,21 +380,20 @@ int  AES_OSSL_##BITCHAIN##_DecryptX2(const unsigned char* ctx, unsigned int roun
 	int olen, elen, ores;					\
 	int rlen = (len&15)? len+16-(len&15): len;		\
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;	\
+	if (IV) {						\
+		SET_IV(evpctx[1], iv, 16, 0);			\
+		SET_IV(evpctx[0], iv, 16, 0);			\
+	}							\
 	EVP_CIPHER_CTX_set_padding(evpctx[1], 0);		\
 	EVP_CIPHER_CTX_set_padding(evpctx[0], pad==PAD_ASNEEDED? 0: pad);	\
-	if (IV) {						\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[1]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[1]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_original_iv(evpctx[0]), iv, 16);	\
-		memcpy(EVP_CIPHER_CTX_iv_noconst(evpctx[0]), iv, 16);	\
-	}							\
+	if (!len && pad != PAD_ALWAYS) { *flen = 0; return 0; }	\
 	ores = EVP_DecryptUpdate(evpctx[1], out, &olen, in, rlen);	\
 	assert(ores);						\
 	ores = EVP_DecryptFinal(evpctx[1], out+olen, &elen);	\
 	assert(ores);						\
 	if (pad == PAD_ASNEEDED) {				\
 		int ilen = olen, olen1;				\
-		uchar buf[16];					\
+		uchar *buf = crypto->blkbuf3;			\
 		ores = EVP_DecryptUpdate(evpctx[0], out, &olen, out, ilen-16);	\
 		assert(ores); assert(olen == ilen-16);		\
 		/* Save piece that gets overwritten */		\
@@ -384,21 +413,23 @@ int  AES_OSSL_##BITCHAIN##_DecryptX2(const unsigned char* ctx, unsigned int roun
 			ores = EVP_DecryptFinal(evpctx[0], out+olen, &elen);	\
 			assert(ores);				\
 		}						\
+		/* if (elen) *(out+olen+elen) = 16-elen; */	\
 		EVP_CIPHER_CTX_free(ctx2);			\
-		asm("":::"memory");				\
+		LFENCE;						\
 	} else {						\
 		ores = EVP_DecryptUpdate(evpctx[0], out, &olen, out, olen+elen);\
 		assert(ores);					\
 		ores = EVP_DecryptFinal(evpctx[0], out+olen, &elen);	\
+		/* if (elen) *(out+olen+elen) = 16-elen; */	\
 	}							\
 	if (pad)						\
 		*flen = olen+elen;				\
 	else							\
 		*flen = len;					\
 	if (IV)							\
-		memcpy(iv, EVP_CIPHER_CTX_iv(evpctx[0]), 16);	\
+		GET_IV(evpctx[0], iv, 16);			\
 	if (pad == PAD_ASNEEDED)				\
-		return (elen? 16-elen: 1);			\
+		return (elen? 16-elen: ILLEGAL_PADDING);	\
 	return ores - 1;					\
 }
 
@@ -441,22 +472,22 @@ void AES_OSSL_Blk_EncryptX2(const unsigned char *ctx, unsigned int rounds,
 {
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;
 	int olen;
-	uchar blk[16];
+	uchar *blk = crypto->blkbuf1;
 	EVP_EncryptUpdate(evpctx[0], blk, &olen, in, 16);
 	EVP_EncryptUpdate(evpctx[1], out, &olen, blk, olen);
 	memset(blk, 0, 16);
-	asm("":::"memory");
+	LFENCE;
 }
 void AES_OSSL_Blk_DecryptX2(const unsigned char *ctx, unsigned int rounds,
 			    const unsigned char *in, unsigned char *out)			
 {
 	EVP_CIPHER_CTX **evpctx = (EVP_CIPHER_CTX**)ctx;
 	int olen;
-	uchar blk[16];
+	uchar *blk = crypto->blkbuf1;
 	EVP_DecryptUpdate(evpctx[1], blk, &olen, in, 16);
 	EVP_DecryptUpdate(evpctx[0], out, &olen, blk, olen);
 	memset(blk, 0, 16);
-	asm("":::"memory");
+	LFENCE;
 }
 
 
