@@ -351,8 +351,8 @@ int main(int argc, char *argv[])
 		have_sse2, have_sse42, have_aesni, have_rdrand, have_sha256, have_avx2, have_vaes);
 #elif defined(__arm__) || defined(__aarch64__)
 	//have_arm8crypto = 1;
-	printf("CPU Features: AES Arm8 %i SHA Arm8 %i\n",
-		have_arm8crypto, have_arm8sha);
+	printf("CPU Features: AES Arm8 %i SHA Arm8 %i RNDR Arm8 %i\n",
+		have_arm8crypto, have_arm8sha, have_arm8rndr);
 #endif
 	crypto = secmem_init();
 	/*
